import { actions, actionTypes as fetchActions } from '@bufferapp/async-data-fetch';
import { actions as analyticsActions } from '@bufferapp/analytics-middleware';
import middleware from './middleware';
import { actionTypes } from './actions';

describe('middleware', () => {
  const next = jest.fn();
  let state = null;

  const store = {
    dispatch: jest.fn(),
    getState: jest.fn(() => state),
  };

  beforeEach(() => {
    state = {
      appSidebar: {
        user: {
          id: 'user1234',
        },
      },
      profiles: {
        selectedProfileId: '12359182129asd',
        organizationId: 'organization1234',
        selectedProfile: { service: 'instagram' },
      },
      date: {
        presets: [{
          range: 7,
          selected: true,
        }],
        startDate: '04/12/2019',
        endDate: '04/18/2019',
      },
    };

    store.dispatch = jest.fn();
  });

  it('should exist', () => {
    expect(middleware).toBeDefined();
  });

  it('should keep propagating the action through the chain', () => {
    const action = {
      type: 'TEST',
    };
    middleware(store)(next)(action);
  });

  it('should create a report when CREATE_REPORT is triggered', () => {
    const action = {
      type: actionTypes.CREATE_REPORT,
      chart_id: 'summary-table',
      name: 'Weekly Sync Report',
    };
    middleware(store)(next)(action);
    expect(store.dispatch).toHaveBeenLastCalledWith(actions.fetch({
      name: 'create_report',
      args: {
        userId: 'user1234',
        profileId: '12359182129asd',
        organizationId: 'organization1234',
        chartId: 'summary-table',
        name: 'Weekly Sync Report',
        dateRange: {
          range: 7,
          start: '04/12/2019',
          end: '04/18/2019',
        },
      },
    }));
    expect(next).toHaveBeenCalledWith(action);
  });

  it(`should track a Report Created event when create_report_${fetchActions.FETCH_SUCCESS} is triggered`, () => {
    analyticsActions.trackEvent = jest.fn();

    const action = {
      type: `create_report_${fetchActions.FETCH_SUCCESS}`,
      result: {
        _id: 'report123',
        name: 'Weekly Sync Report',
      },
    };
    middleware(store)(next)(action);
    expect(analyticsActions.trackEvent)
      .toHaveBeenCalledWith('Report Created', {
        title: 'Weekly Sync Report',
        reportId: 'report123',
        days: 7,
      });
    expect(next).toHaveBeenCalledWith(action);
  });

  it('should add a chart to a report when ADD_TO_REPORT is triggered', () => {
    const action = {
      type: actionTypes.ADD_TO_REPORT,
      chart_id: 'summary-table',
      reportId: 'report-123',
      state: {},
    };
    middleware(store)(next)(action);
    expect(store.dispatch).toHaveBeenCalledWith(actions.fetch({
      name: 'add_chart_to_report',
      args: {
        reportId: 'report-123',
        profileId: '12359182129asd',
        chartId: 'summary-table',
        state: {},
      },
    }));
    expect(next).toHaveBeenCalledWith(action);
  });

  it('should track a Report Module Added event when ADD_TO_REPORT is triggered', () => {
    analyticsActions.trackEvent = jest.fn();

    const action = {
      type: actionTypes.ADD_TO_REPORT,
      chart_id: 'summary-table',
      reportId: 'report-123',
      state: {},
    };
    middleware(store)(next)(action);
    expect(analyticsActions.trackEvent)
      .toHaveBeenCalledWith('Report Module Added', {
        reportId: 'report-123',
        channel: 'instagram',
        module: 'summary table',
      });
    expect(next).toHaveBeenCalledWith(action);
  });

  it('should not store the profileId when creating a report with comparison chart', () => {
    const action = {
      type: actionTypes.CREATE_REPORT,
      chart_id: 'comparison',
      name: 'Weekly Sync Report',
    };
    middleware(store)(next)(action);
    expect(store.dispatch).toHaveBeenLastCalledWith(actions.fetch({
      name: 'create_report',
      args: {
        organizationId: 'organization1234',
        userId: 'user1234',
        profileId: null,
        chartId: 'comparison',
        name: 'Weekly Sync Report',
        dateRange: {
          range: 7,
          start: '04/12/2019',
          end: '04/18/2019',
        },
      },
    }));
    expect(next).toHaveBeenCalledWith(action);
  });

  it('should not store the profileId when adding a comparison chart', () => {
    const action = {
      type: actionTypes.ADD_TO_REPORT,
      chart_id: 'comparison',
      reportId: 'report-123',
      state: {},
    };
    middleware(store)(next)(action);
    expect(store.dispatch).toHaveBeenCalledWith(actions.fetch({
      name: 'add_chart_to_report',
      args: {
        reportId: 'report-123',
        profileId: null,
        chartId: 'comparison',
        state: {},
      },
    }));
    expect(next).toHaveBeenCalledWith(action);
  });
});
